#![recursion_limit = "256"]

use custom_request::{ CustomParamRequest, CustomRequest, DoFastApi };

use log::info;
use std::sync::Arc;
use structopt::StructOpt;
use tower_lsp::{LspService, Server};

mod core;
mod completion;
mod definition;
mod diagnostics;
mod format;
mod server;
mod sources;
mod custom_request;

#[cfg(test)]
mod support;
use server::Backend;

#[derive(StructOpt, Debug)]
#[structopt(name = "Digtal LSP", about = "LSP for Digital IDE")]
struct Opt {}

#[tokio::main]
async fn main() {
    let _ = Opt::from_args();
    let log_handle = flexi_logger::Logger::with(flexi_logger::LogSpecification::info())
        .start()
        .unwrap();
    
    info!("launch Digital LSP");

    let stdin = tokio::io::stdin();
    let stdout = tokio::io::stdout();

    // let (service, socket) = LspService::new(|client| Arc::new(Backend::new(client, log_handle)));

    let (service, socket) = LspService::build(|client| Arc::new(Backend::new(client, log_handle)))
        .custom_method("custom/request", CustomRequest)
        .custom_method("custom/paramRequest", CustomParamRequest)
        .custom_method("api/fast", DoFastApi)
        .finish();

    Server::new(stdin, stdout, socket)
        .serve(service)
        .await;
}
